/* match.c
   
   --Demogramm fr die regex-Bibliothek

   Autor: Michael Uelschen, Universitt Gesamthochschule Kassel, 3/98
          uelschen@ite.maschinenbau.uni-kassel.de

   Das Programm erwartet beim Aufruf zwei Parameter <pattern> und
   <string>. <pattern> wird als regulrer Ausdruck interpretiert
   und in <string> nach einem Teil gesucht, der diesem Ausdruck ent-
   spricht. <pattern> mu in Anfhrungsstriche gesetzt werden, da
   ansonsten etwaige Sonderzeichen von der Shell interpretiert werden.

   Beispiel: 

   match "(a)+(b)+" adaabbde

   erzeugt die folgende Ausgabe:

   0 expression is [2,6] aabb
   1 expression is [3,4] a
   2 expression is [5,6] b              

   Die gesuchte Teil befindet hat die Indices [2..6]
   An der Stelle [3..4] befindet sich der zuletzt gefundene
   erste Teilausdruck (a). An der Stelle [5..6] befindet sich
   der zuletzt gefundene zweite Teilausdruck (b).                 */

#include <stdio.h>
#include <regex.h>
#include <string.h>
#include <malloc.h>

#define BUFFERSIZE 256

int main(int argc, char* argv[]) {
  regex_t buffer;		/* Struktur des endlichen Automaten */
  regmatch_t *regs;		/* Struktur mit gefundenen Teilausdrcken */

  char *pattern, *string;

  int nmatch=10;
  int match,compile,eflags=0,i;

  const int errbuf_size=BUFFERSIZE;
  char errbuf[BUFFERSIZE];

  if (argc!=3) {		/* Alles korrekt? */
    printf("usage : %s <pattern> <string>\n",argv[0]);
    return -1;
  }

  pattern=argv[1];
  string=argv[2];

  /* --Initialisierung des Feldes fr die Teilausdrcke. */
  regs=(regmatch_t*)malloc(nmatch*sizeof(regmatch_t));

  /* --Versuche den regulren Ausdruck zu compilieren. */
  compile=regcomp(&buffer,pattern,REG_EXTENDED);
  if (compile!=0) {
    /* --Fehlerbehandlung */
    regerror(compile,&buffer,errbuf,errbuf_size);
    printf("%s.\n",errbuf);
    return -1;
  }
    
  /* --Versuche nun den regulren Ausdruck gegen den String zu matchen. */
  match=regexec(&buffer,string,nmatch,regs,eflags);
  if (match==REG_NOMATCH) {
    /* --Fehlerbehandlung */
    regerror(match,&buffer,errbuf,errbuf_size);
    printf("%s.\n",errbuf);
    return -1;
  }

  /* --Zerlege die Zeichenkette in die (Teil-)Ausdrcke. */
  for(i=0;i<buffer.re_nsub+1;i++) 
    printf("%2d expression is [%d,%d] %.*s\n",i,regs[i].rm_so,regs[i].rm_eo,
	   regs[i].rm_eo-regs[i].rm_so,string+regs[i].rm_so);

  /* --Speicherplatz wieder freigeben. */
  regfree(&buffer);

  return 0;
}

