/* xtclsh.c
   
   --Demogramm fr die readline Bibliothek

   Autor: Michael Uelschen, Universitt Gesamthochschule Kassel, 3/98
          uelschen@ite.maschinenbau.uni-kassel.de

   Das Programm ist eine verbesserte tclsh und untersttzt 
   eine komfortable Befehlszeileneingabe und command completion,
   wenn die Tabulator-Taste gedrckt wird. */

#include <stdio.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <tcl.h>
#include <string.h>

/*
 * The following variable is a special hack that is needed in order for
 * Sun shared libraries to be used for Tcl.
 */

extern int matherr();
int *tclDummyMathPtr = (int *) matherr;

/* --tclCommand
   Feld mit den in Tcl bekannten Befehlen, die bei der 
   Befehlsvervollstndigung angeboten werden soll. */

char* tclCommand[]={
  "after", "append", "array", "break", "case", "catch", "cd", "clock",
  "close", "concat", "continue", "eof", "error", "eval", "exec", "exit",
  "expr", "fblocked", "fconfigure", "file", "fileevent", "flush", "for",
  "foreach", "format", "gets", "glob", "global", "history", "if", "incr",
  "info", "interp", "join", "lappend", "lindex", "linsert", "list", "llength",
  "load", "lrange", "lreplace", "lsearch", "lsort", "open", "package", "pid",
  "proc", "puts", "pwd", "read", "regexp", "regsub", "rename", "return",
  "scan", "seek", "set", "socket", "source", "split", "string", "subst",
  "switch", "tell", "time", "trace", "unset", "unsupported0", "update",
  "uplevel", "upvar", "vwait", "while", (char*)NULL
};

/* --isLineComplete
   berprfung, ob die bisher eingegebene Zeile vollstndig ist oder ob
   weitere Zeichen notwendig sind (wg. geffneter Klammern) */

int isLineComplete(char* line, int* braces) {
  int i;
  if (!line)
    return 0;
  for(i=0;i<strlen(line);i++)
    if (line[i]=='{')		/* Geffnete Klammer -> Zhler erhhen */
      (*braces)++;
    else if (line[i]=='}')	/* Geschlossene Klammer -> Zhler erniedrigen*/
      (*braces)--;
  return (*braces)==0;
}

/* --executeLine
   Eine Zeichenkette wird als Tcl Befehl ausgefhrt. Handelt es sich
   um eine Fortsetzungszeile, wird der Puffer zustzlich mit ausgwertet. */

int executeLine(Tcl_Interp* interp,char* line,char* buf) {
  int result;
  /* --berprfe, ob einzelne Zeile oder eine Fortsezungszeile ausgewertet
     werden soll. */
  if (strlen(buf)>0)
    result=Tcl_VarEval(interp,buf,line,(char*)NULL);
  else 
    result=Tcl_Eval(interp,line);
  return result;
}

/* --helpOnTclCommand 
 */

int helpOnTclCommand(int count, int key) {
  printf("\n->help on tcl command<-\n");
  /* Hier kann dann irgendeine sinnvolle Hilfefunktion gestartet werden. */
  rl_on_new_line();
  return 0;
}

/* --commandForCompletion
   Es wird versucht, den Inhalt von text zu vervollstndigen. Ist die
   Variable state 0, so wird der Vervollstndigungsproze von Beginn
   neu gestartet. */

char* commandForCompletion(char* text,int state) {
  static int length,counter;
  if (state==0) {		/* Neuinitialisierung */
    length=strlen(text);
    counter=0;
  }
  while (tclCommand[counter])	/* Durchsuche Feld mit mglichen Befehlen */
    if (strncmp(tclCommand[counter],text,length)==0)
      return strdup(tclCommand[counter++]);
    else
      counter++;
  /* --Keine weiteren Befehle mehr mglich. */
  return (char*)NULL;
}

/* --completion
   Ruft die Tcl-Befehlsvervollstngigung auf. text gibt den zu 
   vervollstndigen text an, start und end sind die Indices der
   Zeichenkette im Feld rl_line_buffer. */

char** completion(char* text, int start, int end) {
  return completion_matches(text,commandForCompletion);
}

/* --initializeReadline
   Initialisierung der readline Routinen, Datei+Befehlsvervollstndigung */

void initializeReadline() {

  rl_attempted_completion_function=(CPPFunction*)completion;

  /* --Neue Funktion fr Hilfe ber Tcl-Befehle */
  /*  rl_bind_key('a',helpOnTclCommand); */

  rl_add_defun("help-on-tcl-command",helpOnTclCommand,-1);

  rl_read_init_file("~/.inputrc");

}

int main(int argc,char* argv[]) {
  Tcl_Interp *interp;
  int result;

  char* input=0;
  char* prompt[]={"% ","_ "};

  int braces=0;

  int promptId=0;

  /* --Temporrer Puffer fr Fortsetzungszeilen */
  char buffer[4096];
  buffer[0]='\0';

  /* Readline Funktionalitt initialisieren */
  initializeReadline();

  /* --Interpreter erzeugen */
  interp=Tcl_CreateInterp();

  while(1) {
    /* --Einlesen mit readline Funktion */
    input=readline(prompt[promptId]);
    
    /* --berprfe, ob Zeile vollstndig ist */
    if (input) 
      if (isLineComplete(input,&braces)) {
	braces=0;
	/* --Befehl ausfhren */
	result=executeLine(interp,input,buffer);
	
	/* --berprfe, ob alles korrekt ausgefhrt wurde */
	printf("%s\n",interp->result);
	
	/* --Buffer und Prompt zurcksetzen */
	buffer[0]='\0';
	promptId=0;
      } else {
	/* --Kopiere die Zeile in den Puffer */
	strcat(buffer,input);
	/* --Prompt umsetzen */
	promptId=1;
      }
    
    if (input)
      add_history(input);

    /* Speicher freigeben */
    free(input);
  }



}
